package ru.dbotthepony.mc.otm.block.entity;

import net.minecraft.core.BlockPos;
import net.minecraft.nbt.CompoundTag;
import net.minecraft.network.chat.Component;
import net.minecraft.network.chat.TranslatableComponent;
import net.minecraft.world.entity.player.Inventory;
import net.minecraft.world.entity.player.Player;
import net.minecraft.world.inventory.AbstractContainerMenu;
import net.minecraft.world.level.Level;
import net.minecraft.world.level.block.Block;
import net.minecraft.world.level.block.entity.BlockEntity;
import net.minecraft.world.level.block.entity.BlockEntityType;
import net.minecraft.world.level.block.state.BlockState;
import ru.dbotthepony.mc.otm.Registry;
import ru.dbotthepony.mc.otm.block.BlockDriveViewer;
import ru.dbotthepony.mc.otm.block.entity.worker.WorkerState;
import ru.dbotthepony.mc.otm.capability.MatteryCapability;
import ru.dbotthepony.mc.otm.capability.MatteryMachineEnergyStorage;
import ru.dbotthepony.mc.otm.container.MatteryContainer;
import ru.dbotthepony.mc.otm.menu.DriveViewerMenu;

import javax.annotation.Nullable;
import java.math.BigDecimal;

public class BlockEntityDriveViewer extends BlockEntityMatteryPowered {
	public static final BigDecimal MTE_PER_OPERATION = new BigDecimal("3.125");

	public boolean canIOItems() {
		return energy.getBatteryLevel().compareTo(MTE_PER_OPERATION) >= 0;
	}

	@Override
	public void setChanged() {
		super.setChanged();

		if (level != null) {
			var state = getBlockState();

			if (drive_slot.getItem(0).getCapability(MatteryCapability.DRIVE).isPresent()) {
				if (!getBlockState().getValue(BlockDriveViewer.DRIVE_PRESENT)) {
					state = state.setValue(BlockDriveViewer.DRIVE_PRESENT, true);
				}
			} else {
				if (getBlockState().getValue(BlockDriveViewer.DRIVE_PRESENT)) {
					state = state.setValue(BlockDriveViewer.DRIVE_PRESENT, false);
				}
			}

			if (drive_slot.getItem(0).getCapability(MatteryCapability.DRIVE).isPresent() && canIOItems()) {
				if (getBlockState().getValue(WorkerState.SEMI_WORKER_STATE) != WorkerState.WORKING) {
					state = state.setValue(WorkerState.SEMI_WORKER_STATE, WorkerState.WORKING);
				}
			} else {
				if (getBlockState().getValue(WorkerState.SEMI_WORKER_STATE) != WorkerState.IDLE) {
					state = state.setValue(WorkerState.SEMI_WORKER_STATE, WorkerState.IDLE);
				}
			}

			if (state != getBlockState()) {
				level.setBlock(getBlockPos(), state, Block.UPDATE_CLIENTS);
			}
		}
	}

	public int getIOItemCount(int desired, boolean simulate) {
		if (!canIOItems())
			return 0;

		var extracted = energy.extractEnergyInner(MTE_PER_OPERATION.multiply(new BigDecimal(desired)), true);
		var modulo = extracted.divideToIntegralValue(MTE_PER_OPERATION);

		if (simulate)
			return modulo.intValue();

		energy.extractEnergyInner(modulo.multiply(MTE_PER_OPERATION), false);

		return modulo.intValue();
	}

	public BlockEntityDriveViewer(BlockPos p_155229_, BlockState p_155230_) {
		super(Registry.BlockEntities.DRIVE_VIEWER, p_155229_, p_155230_);
		energy = new MatteryMachineEnergyStorage(this, MatteryMachineEnergyStorage.MachineType.WORKER, new BigDecimal(30_000));
	}

	private static final TranslatableComponent NAME = new TranslatableComponent("block.overdrive_that_matters.drive_viewer");

	public final MatteryContainer drive_slot = new MatteryContainer(this::setChanged, 1);

	@Override
	protected Component getDefaultDisplayName() {
		return NAME;
	}

	@Nullable
	@Override
	public AbstractContainerMenu createMenu(int containerID, Inventory inventory, Player ply) {
		return new DriveViewerMenu(containerID, inventory, this);
	}

	@Override
	public CompoundTag save(CompoundTag nbt) {
		nbt.put("drive_slot", drive_slot.serializeNBT());
		return super.save(nbt);
	}

	@Override
	public void load(CompoundTag nbt) {
		super.load(nbt);
		drive_slot.deserializeNBT(nbt.get("drive_slot"));
	}

	public static <T extends BlockEntity> void ticker(Level level, BlockPos blockPos, BlockState blockState, T t) {
		if (t instanceof BlockEntityDriveViewer tile) {
			tile.batteryChargeLoop();
		}
	}
}
