package ru.dbotthepony.mc.otm.capability;

import net.minecraft.nbt.CompoundTag;
import net.minecraft.nbt.ListTag;
import net.minecraft.nbt.Tag;
import net.minecraft.resources.ResourceLocation;
import net.minecraft.world.entity.Entity;
import net.minecraft.world.entity.player.Player;
import net.minecraftforge.client.event.InputUpdateEvent;
import net.minecraftforge.common.util.LazyOptional;
import net.minecraftforge.event.AttachCapabilitiesEvent;
import net.minecraftforge.event.entity.player.PlayerEvent;
import net.minecraftforge.eventbus.api.SubscribeEvent;
import ru.dbotthepony.mc.otm.OverdriveThatMatters;
import ru.dbotthepony.mc.otm.Registry;
import ru.dbotthepony.mc.otm.android.AndroidResearch;
import ru.dbotthepony.mc.otm.android.AndroidResearchType;
import ru.dbotthepony.mc.otm.network.android.AndroidResearchPacket;
import ru.dbotthepony.mc.otm.network.android.AndroidStatusPacket;

import javax.annotation.Nonnull;
import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.Objects;

public class AndroidCapabilityPlayer extends AndroidCapability {
	public boolean is_android = false;
	private boolean network_is_android = false;

	private final ArrayList<AndroidResearch> research_list = new ArrayList<>();

	@Override
	public void invalidateNetworkState() {
		super.invalidateNetworkState();
		network_is_android = false;

		for (var instance : research_list) {
			instance.dirty = true;
		}
	}

	@Override
	public boolean isAndroid() {
		return is_android;
	}

	public void becomeAndroid() {
		if (is_android)
			return;

		is_android = true;
		energy_stored = new BigDecimal(60_000);
		energy_stored_max = new BigDecimal(60_000);

		ply.setHealth(0.0F);
		ply.die(Registry.DAMAGE_BECOME_ANDROID);
	}

	public void becomeHumane() {
		if (!is_android)
			return;

		is_android = false;

		energy_stored = new BigDecimal(0);
		energy_stored_max = new BigDecimal(32_000);

		dropBattery();

		ply.setHealth(0.0F);
		ply.die(Registry.DAMAGE_BECOME_HUMANE);
	}

	@Override
	public void deserializeNBT(CompoundTag compound) {
		super.deserializeNBT(compound);

		if (compound.contains("is_android"))
			is_android = compound.getByte("is_android") > 0;

		research_list.clear();

		var list = compound.getList("research", Tag.TAG_COMPOUND);

		for (var _tag : list) {
			if (_tag instanceof CompoundTag tag) {
				var research = Registry.ANDROID_RESEARCH().getValue(new ResourceLocation(tag.getString("id")));

				if (research != null) {
					var instance = research.factory(this);
					instance.deserializeNBT(tag);
					research_list.add(instance);
				}
			}
		}
	}

	@Nonnull
	@Override
	public CompoundTag serializeNBT() {
		CompoundTag tag = super.serializeNBT();
		tag.putByte("is_android", is_android ? (byte) 1 : 0);

		var list = new ListTag();

		for (var instance : research_list) {
			var _tag = new CompoundTag();
			instance.serializeNBT(_tag);
			_tag.putString("id", Objects.requireNonNull(instance.type.getRegistryName()).toString());
			list.add(_tag);
		}

		tag.put("research", list);

		return tag;
	}

	public <T extends AndroidResearch> T getResearch(AndroidResearchType<T> type) {
		for (var instance : research_list) {
			if (instance.type == type) {
				return (T) instance;
			}
		}

		var instance = type.factory(this);
		research_list.add(instance);
		return instance;
	}

	@SubscribeEvent
	public static void onAttachCapabilityEvent(AttachCapabilitiesEvent<Entity> event) {
		if (event.getObject() instanceof Player ply) {
			event.addCapability(Registry.Names.ANDROID_CAPABILITY, new AndroidCapabilityPlayer(ply));
		}
	}

	@SubscribeEvent
	public static void onPlayerChangeDimensionEvent(PlayerEvent.PlayerChangedDimensionEvent event) {
		event.getPlayer().getCapability(MatteryCapability.ANDROID).ifPresent(IAndroidCapability::invalidateNetworkState);
	}

	@SubscribeEvent
	public static void onPlayerCloneEvent(PlayerEvent.Clone event) {
		event.getPlayer().getCapability(MatteryCapability.ANDROID).ifPresent((cap) -> {
			LazyOptional<IAndroidCapability> resolver = event.getOriginal().getCapability(MatteryCapability.ANDROID);

			if (!resolver.isPresent() || resolver.resolve().isEmpty()) {
				event.getOriginal().reviveCaps();
				resolver = event.getOriginal().getCapability(MatteryCapability.ANDROID);
			}

			if (!resolver.isPresent() || resolver.resolve().isEmpty()) {
				event.getOriginal().invalidateCaps();
				return;
			}

			cap.deserializeNBT(resolver.resolve().get().serializeNBT());
			cap.invalidateNetworkState();
			event.getOriginal().invalidateCaps();
		});
	}

	public int last_jump_ticks = 14;
	public final Player ply;

	public AndroidCapabilityPlayer(Player ent) {
		super(ent);
		ply = ent;
	}

	@Override
	protected void tickNetwork() {
		super.tickNetwork();

		if (is_android != network_is_android) {
			network_is_android = is_android;
			sendNetwork(new AndroidStatusPacket(is_android));
		}

		for (var instance : research_list) {
			if (instance.dirty) {
				instance.dirty = false;
				sendNetwork(new AndroidResearchPacket(instance));
			}
		}
	}

	public static final BigDecimal ENERGY_FOR_HUNGER_POINT = new BigDecimal(1000);

	@Override
	public void tickServer() {
		super.tickServer();

		// TODO: Maybe passive drain?
		// extractEnergyInner(BigDecimal.valueOf(new Random().nextDouble()), false);

		if (ply.isSwimming() && !hasFeature(Registry.AndroidFeatures.AIR_BAGS)) {
			ply.setSwimming(false);
		}

		var stats = ply.getFoodData();

		while (stats.getFoodLevel() < 18 && this.extractEnergyInner(ENERGY_FOR_HUNGER_POINT, true).compareTo(ENERGY_FOR_HUNGER_POINT) == 0) {
			this.extractEnergyInner(ENERGY_FOR_HUNGER_POINT, false);
			stats.setFoodLevel(stats.getFoodLevel() + 1);
		}

		// "block" quick regeneration
		// also cause power to generate while in peaceful
		while (stats.getFoodLevel() > 18 && this.receiveEnergyInner(ENERGY_FOR_HUNGER_POINT, true).compareTo(ENERGY_FOR_HUNGER_POINT) == 0) {
			this.receiveEnergyInner(ENERGY_FOR_HUNGER_POINT, false);
			stats.setFoodLevel(stats.getFoodLevel() - 1);
		}

		var food_level = (float) stats.getFoodLevel();

		if (stats.getSaturationLevel() < food_level) {
			BigDecimal extracted = this.extractEnergyInner(ENERGY_FOR_HUNGER_POINT.multiply(new BigDecimal(Float.toString(food_level - stats.getSaturationLevel()))), false);
			stats.setSaturation(stats.getSaturationLevel() + extracted.divide(ENERGY_FOR_HUNGER_POINT, MatteryCapability.ROUND_RULES).floatValue());
		}

		if (stats.getExhaustionLevel() > 0f) {
			BigDecimal extracted = this.extractEnergyInner(ENERGY_FOR_HUNGER_POINT.multiply(new BigDecimal(Float.toString(stats.getExhaustionLevel() / 4f))), false);
			stats.setExhaustion(stats.getExhaustionLevel() - extracted.divide(ENERGY_FOR_HUNGER_POINT, MatteryCapability.ROUND_RULES).floatValue() * 4f);
		}
	}
}
