package ru.dbotthepony.mc.otm.client.render;

import com.mojang.blaze3d.systems.RenderSystem;

public record RGBAColor(float r, float g, float b, float a) {
	public RGBAColor(float r, float g, float b, float a) {
		this.r = r;
		this.g = g;
		this.b = b;
		this.a = a;
	}

	public static final RGBAColor BLACK = new RGBAColor(0, 0, 0, 1f);
	public static final RGBAColor WHITE = new RGBAColor(1f, 1f, 1f, 1f);

	public RGBAColor(int r, int g, int b) {
		this(r / 255F, g / 255F, b / 255F, 1F);
	}

	public RGBAColor(int r, int g, int b, int a) {
		this(r / 255F, g / 255F, b / 255F, a / 255F);
	}

	public static RGBAColor inv(int color) {
		var r = ((color & 0xFF000000) >>> 24) / 255F;
		var g = ((color & 0xFF0000) >>> 16) / 255F;
		var b = ((color & 0xFF00) >>> 8) / 255F;
		var a = (color & 0xFF) / 255F;

		return new RGBAColor(a, b, g, r);
	}

	public RGBAColor(int color) {
		this(((color & 0xFF000000) >>> 24) / 255F, ((color & 0xFF0000) >>> 16) / 255F, ((color & 0xFF00) >>> 8) / 255F, (color & 0xFF) / 255F);
	}

	public int toInt() {
		var r = (int) this.r;
		var g = (int) this.g;
		var b = (int) this.b;

		return r << 16 | g << 8 | b;
	}

	public void setSystemColor() {
		setShaderColor();
		setDrawColor();
	}

	public void setShaderColor() {
		RenderSystem.setShaderColor(r, g, b, a);
	}

	public void setDrawColor() {
		RenderHelper.setDrawColor(this);
	}
}
