package ru.dbotthepony.mc.otm.client.screen;

import net.minecraft.ChatFormatting;
import net.minecraft.client.gui.components.EditBox;
import net.minecraft.network.chat.Component;
import net.minecraft.network.chat.TextComponent;
import net.minecraft.network.chat.TranslatableComponent;
import net.minecraft.resources.ResourceLocation;
import net.minecraft.world.entity.player.Inventory;
import net.minecraft.world.item.ItemStack;
import ru.dbotthepony.mc.otm.OverdriveThatMatters;
import ru.dbotthepony.mc.otm.capability.MatterTask;
import ru.dbotthepony.mc.otm.capability.PatternState;
import ru.dbotthepony.mc.otm.menu.MatterPanelMenu;
import ru.dbotthepony.mc.otm.network.MatteryNetworking;
import ru.dbotthepony.mc.otm.network.PatternReplicationRequestPacket;
import ru.dbotthepony.mc.otm.client.screen.panels.*;

import javax.annotation.Nonnull;
import javax.annotation.Nullable;
import java.util.ArrayList;
import java.util.List;

public class MatterPanelScreen extends MatteryScreen<MatterPanelMenu> {
	private static final int MODAL_WIDTH = 213;
	private static final int MODAL_HEIGHT = 110;

	protected static final ResourceLocation CONTAINER = new ResourceLocation(OverdriveThatMatters.MOD_ID, "textures/gui/matter_panel.png");

	protected ResourceLocation CONTAINER_BACKGROUND() {
		return CONTAINER;
	}

	public MatterPanelScreen(MatterPanelMenu p_97741_, Inventory p_97742_, Component p_97743_) {
		super(p_97741_, p_97742_, p_97743_);

		imageWidth = 176;
		imageHeight = 187;

		titleLabelY = 5;
	}

	public static final int GRID_WIDTH = 8;
	public static final int GRID_HEIGHT = 9;

	private final ArrayList<AbstractSlotPanel> pattern_slots = new ArrayList<>();
	private final ArrayList<AbstractSlotPanel> task_slots = new ArrayList<>();

	@Nullable
	@Override
	protected FramePanel makeMainFrame() {
		float width = GRID_WIDTH * AbstractSlotPanel.REGULAR_DIMENSIONS + ScrollBarPanel.WIDTH + FramePanel.PADDING * 2 + 8;
		float height = GRID_HEIGHT * AbstractSlotPanel.REGULAR_DIMENSIONS + FramePanel.PADDING_TOP + FramePanel.PADDING;

		var scroll_panel = new ScrollBarPanel(this, null, 0, 0, 0);

		var frame = new FramePanel(this, null, 0, 0, width, height, getTitle()) {
			@Override
			protected boolean mouseScrolledInner(double mouse_x, double mouse_y, double scroll) {
				return scroll_panel.mouseScrolledInner(mouse_x, mouse_y, scroll);
			}
		};

		var patterns_tab = frame.addTab(FramePanel.FrameTabPosition.TOP, () -> {
			for (var slot : pattern_slots) {
				slot.setVisible(true);
			}
		}, () -> {
			for (var slot : pattern_slots) {
				slot.setVisible(false);
			}
		});

		var tasks_tab = frame.addTab(FramePanel.FrameTabPosition.TOP, () -> {
			for (var slot : task_slots) {
				slot.setVisible(true);
			}
		}, () -> {
			for (var slot : task_slots) {
				slot.setVisible(false);
			}
		});

		scroll_panel.setParent(frame);
		scroll_panel.setDock(Dock.RIGHT);
		scroll_panel.setupRowMultiplier(() -> {
			if (tasks_tab.isActive()) {
				return menu.tasks.size() / GRID_WIDTH;
			}

			return menu.patterns.size() / GRID_WIDTH;
		});

		var grid = new GridPanel(this, frame, 0, 0, GRID_WIDTH * AbstractSlotPanel.REGULAR_DIMENSIONS, 0, GRID_WIDTH, GRID_HEIGHT) {
			@Override
			protected boolean mouseScrolledInner(double mouse_x, double mouse_y, double scroll) {
				return scroll_panel.mouseScrolledInner(mouse_x, mouse_y, scroll);
			}
		};

		grid.setDock(Dock.LEFT);
		grid.setDockMargin(4, 0, 0, 0);

		for (int i = 0; i < GRID_WIDTH * GRID_HEIGHT; i++) {
			int slot = i;

			pattern_slots.add(new AbstractSlotPanel(this, grid, 0, 0) {
				@Nonnull
				@Override
				protected ItemStack getItemStack() {
					var slot1 = slot + scroll_panel.getScroll(menu.patterns.size() / GRID_WIDTH) * GRID_WIDTH;

					if (slot1 >= menu.patterns.size()) {
						return ItemStack.EMPTY;
					}

					return menu.patterns.get(slot1).stack();
				}

				@Override
				protected List<Component> getItemStackTooltip(ItemStack stack) {
					var slot1 = slot + scroll_panel.getScroll(menu.patterns.size() / GRID_WIDTH) * GRID_WIDTH;

					if (slot1 >= menu.patterns.size()) {
						return List.of();
					}

					return getPatternTooltip(super.getItemStackTooltip(stack), menu.patterns.get(slot1));
				}

				@Override
				protected boolean mouseScrolledInner(double mouse_x, double mouse_y, double scroll) {
					return scroll_panel.mouseScrolledInner(mouse_x, mouse_y, scroll);
				}

				@Override
				protected boolean mouseClickedInner(double mouse_x, double mouse_y, int flag) {
					if (slot >= menu.patterns.size()) {
						return true;
					}

					openPattern(menu.patterns.get(slot));

					return true;
				}
			});

			task_slots.add(new AbstractSlotPanel(this, grid, 0, 0) {
				@Nonnull
				@Override
				protected ItemStack getItemStack() {
					var slot1 = slot + scroll_panel.getScroll(menu.tasks.size() / GRID_WIDTH) * GRID_WIDTH;

					if (slot1 >= menu.tasks.size()) {
						return ItemStack.EMPTY;
					}

					var task = menu.tasks.get(slot1);
					return task.stack(Math.max(task.required(), 1));
				}

				@Override
				protected List<Component> getItemStackTooltip(ItemStack stack) {
					var slot1 = slot + scroll_panel.getScroll(menu.tasks.size() / GRID_WIDTH) * GRID_WIDTH;

					if (slot1 >= menu.tasks.size()) {
						return List.of();
					}

					return getTaskTooltip(super.getItemStackTooltip(stack), menu.tasks.get(slot1));
				}

				@Override
				protected boolean mouseScrolledInner(double mouse_x, double mouse_y, double scroll) {
					return scroll_panel.mouseScrolledInner(mouse_x, mouse_y, scroll);
				}

				@Override
				protected boolean mouseClickedInner(double mouse_x, double mouse_y, int flag) {
					if (slot >= menu.tasks.size()) {
						return true;
					}

					openTask(menu.tasks.get(slot));

					return true;
				}
			});
		}

		for (var slot : task_slots) {
			slot.setVisible(false);
		}

		return frame;
	}

	private List<Component> getTaskTooltip(List<Component> input, MatterTask task) {
		input.add(new TranslatableComponent("otm.gui.matter_task.total", task.total()).withStyle(ChatFormatting.GRAY));
		input.add(new TranslatableComponent("otm.gui.matter_task.required", task.required()).withStyle(ChatFormatting.GRAY));
		input.add(new TranslatableComponent("otm.gui.matter_task.in_progress", task.in_progress()).withStyle(ChatFormatting.GRAY));
		input.add(new TranslatableComponent("otm.gui.matter_task.finished", task.finished()).withStyle(ChatFormatting.GRAY));

		return input;
	}

	private List<Component> getPatternTooltip(List<Component> input, PatternState pattern) {
		input.add(new TranslatableComponent("otm.item.pattern.research", String.format("%.2f", pattern.research_percent() * 100d)).withStyle(ChatFormatting.AQUA));

		return input;
	}

	private void openTask(MatterTask task) {
		var task_frame = new FramePanel(this, null, 0, 0, 170, 40, new TranslatableComponent("otm.container.matter_panel.task")) {
			@Override
			public void tick() {
				super.tick();

				if (!menu.tasks.contains(task)) {
					remove();
				}
			}
		};

		var slot = new AbstractSlotPanel(this, task_frame, 0, 0) {
			@Nonnull
			@Override
			protected ItemStack getItemStack() {
				var task1_index = menu.tasks.indexOf(task);

				if (task1_index != -1) {
					var task1 = menu.tasks.get(task1_index);
					return task1.stack(Math.max(task1.required(), 1));
				}

				return ItemStack.EMPTY;
			}

			@Override
			protected List<Component> getItemStackTooltip(ItemStack stack) {
				var task1_index = menu.tasks.indexOf(task);
				List<Component> get_list = super.getItemStackTooltip(stack);

				if (task1_index != -1) {
					getTaskTooltip(get_list, menu.tasks.get(task1_index));
				}

				return get_list;
			}
		};

		slot.setDock(Dock.LEFT);

		var button = new ButtonPanel(this, task_frame, 0, 0, 40, 20, new TranslatableComponent("otm.container.matter_panel.close"));
		button.setDock(Dock.RIGHT);
		button.setDockMargin(2, 0, 0, 0);
		button.bindOnPress(task_frame::remove);

		button = new ButtonPanel(this, task_frame, 0, 0, 80, 20, new TranslatableComponent("otm.container.matter_panel.cancel_task"));
		button.setDock(Dock.RIGHT);
		button.setDockMargin(2, 0, 0, 0);
		button.bindOnPress(() -> {
			menu.requestTaskCancel(task.id());
			task_frame.remove();
		});

		task_frame.toScreenCenter();
		addPanel(task_frame);
	}

	private void openPattern(PatternState state) {
		var pattern_frame = new FramePanel(this, null, 0, 0, MODAL_WIDTH, MODAL_HEIGHT, new TranslatableComponent("otm.container.matter_panel.label")){
			@Override
			public void tick() {
				super.tick();

				if (!menu.patterns.contains(state)) {
					remove();
				}
			}
		};

		var row_1 = new EditablePanel(this, pattern_frame);
		var row_2 = new EditablePanel(this, pattern_frame);
		var row_3 = new EditablePanel(this, pattern_frame);
		var row_4 = new EditablePanel(this, pattern_frame);

		row_1.setDock(Dock.TOP);
		row_2.setDock(Dock.TOP);
		row_3.setDock(Dock.TOP);
		row_4.setDock(Dock.TOP);

		row_1.setHeight(20);
		row_2.setHeight(20);
		row_3.setHeight(20);
		row_4.setHeight(20);

		row_1.setDockMargin(0, 2, 0, 0);
		row_2.setDockMargin(0, 2, 0, 0);
		row_3.setDockMargin(0, 2, 0, 0);
		row_4.setDockMargin(0, 2, 0, 0);

		var slot = new AbstractSlotPanel(this, row_2, 0, 0) {
			@Nonnull
			@Override
			protected ItemStack getItemStack() {
				return new ItemStack(state.item(), 1);
			}

			@Override
			protected List<Component> getItemStackTooltip(ItemStack stack) {
				return getPatternTooltip(super.getItemStackTooltip(stack), state);
			}
		};

		var input_amount = new EditBoxPanel(this, row_2, 0, 0, 10, 20, new TextComponent("Input amount")) {
			@Override
			protected void factorize(EditBox widget, boolean recreation) {
				super.factorize(widget, recreation);

				widget.setMaxLength(6);

				if (!recreation) {
					widget.setValue("1");
				}
			}

			private void increase(int amount) {
				int value = 1;

				try {
					value = Integer.parseInt(getOrCreateWidget().getValue());
				} catch (Exception ignored) {

				}

				if (value == 1 && amount > 0)
					getOrCreateWidget().setValue(Integer.toString(amount));
				else
					getOrCreateWidget().setValue(Integer.toString(Math.max(1, Math.min(99999, value + amount))));
			}
		};

		var button = new ButtonPanel(this, row_1, 0, 0, 40, 20, new TranslatableComponent("otm.container.matter_panel.increase_by", 8));
		button.bindOnPress(() -> input_amount.increase(8));
		button.setDock(Dock.RIGHT);
		button.setDockMargin(2, 0, 0, 0);

		button = new ButtonPanel(this, row_1, 0, 0, 40, 20, new TranslatableComponent("otm.container.matter_panel.increase_by", 64));
		button.setDock(Dock.RIGHT);
		button.bindOnPress(() -> input_amount.increase(64));
		button.setDockMargin(2, 0, 0, 0);

		button = new ButtonPanel(this, row_1, 0, 0, 40, 20, new TranslatableComponent("otm.container.matter_panel.increase_by", 256));
		button.setDock(Dock.RIGHT);
		button.bindOnPress(() -> input_amount.increase(256));
		button.setDockMargin(2, 0, 0, 0);

		slot.setDock(Dock.LEFT);
		slot.setDockMargin(0, 0, 4, 0);
		input_amount.setDock(Dock.FILL);

		button = new ButtonPanel(this, row_3, 0, 0, 40, 20, new TranslatableComponent("otm.container.matter_panel.decrease_by", 8));
		button.bindOnPress(() -> input_amount.increase(-8));
		button.setDock(Dock.RIGHT);
		button.setDockMargin(2, 0, 0, 0);

		button = new ButtonPanel(this, row_3, 0, 0, 40, 20, new TranslatableComponent("otm.container.matter_panel.decrease_by", 64));
		button.setDock(Dock.RIGHT);
		button.bindOnPress(() -> input_amount.increase(-64));
		button.setDockMargin(2, 0, 0, 0);

		button = new ButtonPanel(this, row_3, 0, 0, 40, 20, new TranslatableComponent("otm.container.matter_panel.decrease_by", 256));
		button.setDock(Dock.RIGHT);
		button.bindOnPress(() -> input_amount.increase(-256));
		button.setDockMargin(2, 0, 0, 0);

		button = new ButtonPanel(this, row_4, 0, 0, 40, 20, new TranslatableComponent("otm.container.matter_panel.cancel"));
		button.setDock(Dock.RIGHT);
		button.bindOnPress(pattern_frame::remove);
		button.setDockMargin(2, 0, 0, 0);

		button = new ButtonPanel(this, row_4, 0, 0, 82, 20, new TranslatableComponent("otm.container.matter_panel.send"));
		button.setDock(Dock.RIGHT);
		button.bindOnPress(() -> {
			int value = 1;

			try {
				value = Integer.parseInt(input_amount.getOrCreateWidget().getValue());
			} catch (Exception ignored) {

			}

			MatteryNetworking.CHANNEL.sendToServer(new PatternReplicationRequestPacket(state, value));
			pattern_frame.remove();
		});

		button.setDockMargin(2, 0, 0, 0);

		addPanel(pattern_frame);
		pattern_frame.toScreenCenter();
		popup(pattern_frame);
	}
}
